{% extends "conneg/base.html" %}

{% block title %}Not Found{% endblock %}

{% block content %}
  <h1>Not Found</h1>
  
  <p>The requested resource does not exist.</p>

  {% if error.message %}<p>{{ error.message }}</p>{% endif %}
{% endblock %}